﻿using System.Collections.Generic;
using System.Globalization;
using System.IO;
using Microsoft.Data.Schema.ScriptDom;
using Microsoft.Data.Schema.SchemaModel;

namespace MySql.Dsp.ScriptDom
{
    /// <summary>
    /// Used to parse MySql scripts.  It can be instantiated directly, or from 
    /// the DatatabaseSchemaProvider using the SqlCeParserConstructor.     
    /// 
    /// A successful parse will result in Script DOM objects in the form of
    /// IScriptFragment implementing objects.
    /// </summary>
    internal sealed class MySqlParser : Parser
    {
        public MySqlParser()
        {
        }

        private static ParseError CreateParseError(string message, params object[] values)
        {
            return CreateParseError(message, 1, values);
        }

        private static ParseError CreateParseError(string message, int offset, params object [] values)
        {
            ParseError error = new ParseError(SqlCeErrors.SyntaxError, offset - 1, 1, 1, string.Format(CultureInfo.CurrentCulture, message, values));
            return error;
        }

        //private static ParseError CreateParseError(antlr.RecognitionException e)
        //{
        //    ParseError error = null;
        //    int offset = e.getColumn();
            
        //    if (e is antlr.MismatchedCharException)
        //    {
        //        antlr.MismatchedCharException misMatchedCharException = (antlr.MismatchedCharException)e;
        //        error = CreateParseError(SqlCeResources.ParserError_UnexpectedChar, offset, (char)misMatchedCharException.expecting, (char)misMatchedCharException.foundChar);
        //    }
        //    else if (e is antlr.MismatchedTokenException)
        //    {
        //        antlr.MismatchedTokenException misMatchedTokenException = (antlr.MismatchedTokenException)e;
        //        error = CreateParseError(SqlCeResources.ParserError_UnexpectedToken, offset, misMatchedTokenException.tokenNames[misMatchedTokenException.expecting], misMatchedTokenException.token.getText());
        //    }
        //    else if (e is antlr.NoViableAltException)
        //    {
        //        antlr.NoViableAltException noViableAltException = (antlr.NoViableAltException)e;
        //        error = CreateParseError(SqlCeResources.ParserError_InvalidToken, offset, noViableAltException.token.getText());
        //    }
        //    else if (e is antlr.NoViableAltForCharException)
        //    {
        //        antlr.NoViableAltForCharException noViableAltForCharException = (antlr.NoViableAltForCharException)e;
        //        error = CreateParseError(SqlCeResources.ParserError_InvalidChar, offset, noViableAltForCharException.foundChar);
        //    }
        //    else if (e is antlr.SemanticException)
        //    {
        //        error = CreateParseError(SqlCeResources.ParserError_UnexpectedParserError, offset);
        //    }
        //    else 
        //    {
        //        error = CreateParseError(SqlCeResources.ParserError_RecognitionError, offset);
        //    }

        //    return error;
        //}

        /// <summary>
        /// Parses a SqlCe statement and creates a script fragment.  
        /// </summary>
        /// <remarks>
        /// <para>Currently only one statement per input is supported.  Any additional statements result in a syntax error.  
        /// Foreign Key constraints that reference only tables but not columns are also not supported.
        /// </para>
        /// </remarks>
        /// <param name="input">The TextReader from which to read the SqlCe from.</param>
        /// <param name="fragment">The resulting AST fragment, if the script parses successfully.</param>
        /// <param name="errors">A list of syntax errors generated by the parser.</param>
        //public override void Parse(TextReader input, out IScriptFragment fragment, out IList<ParseError> errors)
        public override IScriptFragment  Parse(TextReader input, out IList<ParseError> errors)
        {
            //InternalLexer lexer = new InternalLexer(input);
            //InternalParser parser = new InternalParser(lexer);
            errors = new List<ParseError>();
            IScriptFragment fragment = null;

            //ParseError error = null;

            //try
            //{
            //    fragment = parser.statement();
            //}
            //catch (antlr.TokenStreamIOException)
            //{
            //    error = CreateParseError(SqlCeResources.ParserError_UnexpectedIOError);
            //}
            //catch (antlr.TokenStreamRetryException)
            //{
            //    error = CreateParseError(SqlCeResources.ParserError_UnexpectedParserError);
            //}
            //catch (antlr.TokenStreamRecognitionException e)
            //{
            //    error = CreateParseError(e.recog);
            //}
            //catch (antlr.TokenStreamException)
            //{
            //    error = CreateParseError(SqlCeResources.ParserError_RecognitionError);
            //}           
            //catch (antlr.RecognitionException e)
            //{
            //    error = CreateParseError(e);
            //}
            //catch (antlr.CharStreamIOException)
            //{
            //    error = CreateParseError(SqlCeResources.ParserError_UnexpectedIOError);
            //}
            //catch (antlr.CharStreamException)
            //{
            //    error = CreateParseError(SqlCeResources.ParserError_UnexpectedIOError);
            //}
            //catch (antlr.ANTLRException)
            //{
            //    error = CreateParseError(SqlCeResources.ParserError_UnexpectedParserError);
            //}
            //catch (SqlCeParserException e)
            //{
            //    error = CreateParseError(e.Message, e.Offset);
            //}

            //if (error != null)
            //{
            //    errors.Add(error);
            //}

            return fragment;
        }

        internal static class SqlCeErrors
        {
            public const string SyntaxError = "SyntaxError";
        }

        #region Constructor 

        internal class Constructor : ParserConstructor
        {
            protected override Parser ConstructServiceImplementation()
            {
                return new MySqlParser();
            }
        }

        #endregion

    }
}
